
"""
%prog

Michael Coughlin (coughlim@carleton.edu)

This program runs seismon_run.py.

"""

# =============================================================================
#
#                               PREAMBLE
#
# =============================================================================


import os, time
import numpy as np

__author__ = "Michael Coughlin <coughlim@carleton.edu>"
__date__ = "2012/2/7"
__version__ = "0.1"


paramsFile = "/home/mcoughlin/Seismon/seismon/input/seismon_params_H1EQMon.txt"
paramsFile = "/home/mcoughlin/Seismon/seismon/input/seismon_params_H1O1.txt"
#paramsFile = "/home/eric.coughlin/gitrepo/seismon/input/seismon_params_H1O1.txt"
paramsFile = "/users/coughlin/Seismon/seismon/input/seismon_params_V1O1O2.txt"

gpsStart = 1126051217
gpsEnd = 1133795277

gpsStart = 1126569617
gpsEnd = 1136678417

gpsStart = 1126073342
gpsEnd = 1137283217  

gpsStart = 1126073342
gpsEnd = 1173972188

print "python seismon_run -p %s -s %d -e %d -c V1:ENV_CEB_SEIS_V_50Hz --doEarthquakes --doEarthquakesAnalysis --doPSD --eventfilesType iris --minMagnitude 4.5"%(paramsFile,gpsStart,gpsEnd)
 
#os.system("python seismon_run -p %s -s %d -e %d -c V1:ENV_CEB_SEIS_V_50Hz --doEarthquakes --doEarthquakesAnalysis --doPSD --eventfilesType iris --minMagnitude 4.5"%(paramsFile,gpsStart,gpsEnd))

os.system("python seismon_run -p %s -s %d -e %d -c V1:ENV_CEB_SEIS_V_50Hz --doEarthquakes --doEarthquakesAnalysis --doPSD --eventfilesType iris --minMagnitude 7.5"%(paramsFile,gpsStart,gpsEnd))


